<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the dashboard.
 *
 * @link 		https://lin3s.com
 * @since 		1.0.0
 *
 * @package 	Alexandria_Mail_Builder
 * @subpackage 	Alexandria_Mail_Builder/includes
 */

class Alexandria_Mail_Builder {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		protected
	 * @var 		Mail_Builder_Loader 		$loader 		Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since 		1.0.0
	 * @access 		protected
	 * @var 		string 			$plugin_name 		The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * Sanitizer for cleaning user input
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      Mail_Builder_Sanitize    $sanitizer    Sanitizes data
	 */
	private $sanitizer;

	/**
	 * The current version of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		protected
	 * @var 		string 			$version 		The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the Dashboard and
	 * the public-facing side of the site.
	 *
	 * @since 		1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'alexandria-mail-builder';
		$this->version = '1.0.0';

		define( 'ALEXANDRIA_MAIL_BUILDER_PATH', dirname( __FILE__ ) );
		define( 'ALEXANDRIA_MAIL_BUILDER_RELATIVE_PATH', dirname( plugin_basename( __FILE__ )));
		define( 'ALEXANDRIA_MAIL_BUILDER_URL', plugin_dir_url( __FILE__ ));
		define( 'ALEXANDRIA_MAIL_BUILDER_PLUGIN_PATH', WP_PLUGIN_DIR."/alexandria-mail-builder/" );

		// start session only on front-end and Alexandria Mail Builder admin pages
		if (!session_id() and (strstr(@$_GET['page'], 'alexandriamailbuilder') or !is_admin() or (wp_doing_ajax() and !empty($_POST['alexandria_mail_builder_action']))) ) {
			@session_start();
		}

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->define_template_hooks();
		$this->define_custom_post_types();

		$this->define_widget_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Alexandria_Mail_Builder_Loader. Orchestrates the hooks of the plugin.
	 * - Alexandria_Mail_Builder_i18n. Defines internationalization functionality.
	 * - Alexandria_Mail_Builder_Admin. Defines all hooks for the dashboard.
	 * - Alexandria_Mail_Builder_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-loader.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-database.php';

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-cpt.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the Dashboard.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/'.$this->plugin_name.'-admin.php';

		/**
		 * The class responsible for defining all settings fields.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/'.$this->plugin_name.'-settings-fields.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/'.$this->plugin_name.'-public.php';

		/**
		 * The class responsible for defining all actions creating the templates.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/'.$this->plugin_name.'-template-functions.php';

		/**
		 * The class responsible for all global functions.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-global-functions.php';

		/**
		 * The class responsible for defining all actions shared by the Dashboard and public-facing sides.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-shared.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-widget.php';

		/**
		 * The class responsible for sanitizing user input
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/'.$this->plugin_name.'-sanitize.php';


		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/metaboxes/'.$this->plugin_name.'-mails-metaboxes.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/metaboxes/'.$this->plugin_name.'-tonos-metaboxes.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/metaboxes/'.$this->plugin_name.'-interlocutors-email-metaboxes.php';

		/**
		 * Controllers
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/MainEmailBuilderController.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/controllers/MainAjaxEmailController.php';
		
		$this->loader = new Alexandria_Mail_Builder_Loader();
		$this->sanitizer = new Alexandria_Mail_Builder_Sanitize();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Alexandria_Mail_Builder_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function set_locale() {

		$plugin_i18n = new Alexandria_Mail_Builder_i18n();
		$plugin_i18n->set_domain( 'alexandria-mail-builder' );

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the dashboard functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_custom_post_types() {

		$plugin_cpt = new Alexandria_Mail_Builder_Cpt( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'init', $plugin_cpt, 'register_custom_post_types' );
		$this->loader->add_action( 'init', $plugin_cpt, 'register_custom_taxonomies' );
		$this->loader->add_action( 'add_meta_boxes', $plugin_cpt, 'define_custom_post_type_mataboxes' );
	}

	/**
	 * Register all of the hooks related to the dashboard functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Alexandria_Mail_Builder_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'load_media_files' );
		
		$this->loader->add_filter( 'plugin_action_links_' . ALEXANDRIA_MAIL_BUILDER_FILE, $plugin_admin, 'link_settings' );

		$this->loader->add_action( 'admin_menu', $plugin_admin, 'add_menu' );

		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_settings' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'register_fields' );

		$this->loader->add_action( 'admin_notices', $plugin_admin, 'display_admin_notices' );
		$this->loader->add_action( 'admin_init', $plugin_admin, 'admin_notices_init' );

		$this->loader->add_action( 'parent_file', $plugin_admin, 'menu_highlight' );

		// Ajax actions
		add_action('wp_ajax_alexandria_email_download_template_ajax', 'alexandria_email_download_template_ajax');
		add_action('wp_ajax_nopriv_alexandria_email_download_template_ajax', 'alexandria_email_download_template_ajax');

		add_action('wp_ajax_alexandria_email_builder_order_list_cpt_ajax', 'alexandria_email_builder_order_list_cpt_ajax');
		add_action('wp_ajax_nopriv_alexandria_email_builder_order_list_cpt_ajax', 'alexandria_email_builder_order_list_cpt_ajax');

		$this->loader->add_filter('use_block_editor_for_post_type', $plugin_admin, 'alexandria_disable_gutenberg', 10, 2);

		$tono_email_metabox = new Alexandria_Mail_Tonos_Metaboxes( $this->plugin_name, $this->version );
		
		$this->loader->add_action( 'save_post', $tono_email_metabox, 'alexandria_tonos_email_save_meta_boxes' );
		$this->loader->add_action( 'admin_head-edit.php', $tono_email_metabox,'add_order_button_on_tonos_email_list_page');
		$this->loader->add_filter( 'manage_edit-tonos_email_sortable_columns', $tono_email_metabox,'order_column_register_sortable');
		$this->loader->add_filter( 'pre_get_posts', $tono_email_metabox,  'set_custom_tonos_email_admin_order', 5);
		$this->loader->add_filter( 'manage_tonos_email_posts_columns',$tono_email_metabox, 'set_custom_edit_tonos_email_columns' );
		$this->loader->add_action( 'manage_tonos_email_posts_custom_column' ,$tono_email_metabox, 'custom_tonos_email_column', 10, 2 );
		$this->loader->add_action( 'after_delete_post', $tono_email_metabox, 'reorder_tonos_email_on_delete' );
		$this->loader->add_action( 'wp_trash_post', $tono_email_metabox, 'reorder_tonos_email_on_send_trash' );

		$emails_metaboxes = new Alexandria_Mail_Mails_Metaboxes( $this->plugin_name, $this->version );

		$this->loader->add_action( 'save_post', $emails_metaboxes, 'alexandria_emails_save_custom_fields' );
		$this->loader->add_action( 'manage_emails_posts_columns', $emails_metaboxes, 'set_custom_edit_emails_columns' );
		$this->loader->add_action( 'manage_emails_posts_custom_column', $emails_metaboxes, 'custom_emails_column', 10, 2 );
		$this->loader->add_action( 'manage_edit-emails_sortable_columns', $emails_metaboxes, 'emails_column_register_sortable', 10, 2 );
		$this->loader->add_action('restrict_manage_posts', $emails_metaboxes, 'tsm_filter_post_type_by_taxonomy');
		$this->loader->add_filter('parse_query', $emails_metaboxes, 'tsm_convert_id_to_term_in_query');

		$interlocutors_emails_metaboxes = new Alexandria_Mail_Interlocutors_Email_Metaboxes( $this->plugin_name, $this->version );

		$this->loader->add_action( 'save_post', $interlocutors_emails_metaboxes, 'alexandria_interlocutors_email_save_meta_boxes' );
		$this->loader->add_action( 'admin_head-edit.php', $interlocutors_emails_metaboxes,'add_order_button_on_interlocutors_email_list_page');
		$this->loader->add_filter( 'manage_edit-interlocutors_email_sortable_columns', $interlocutors_emails_metaboxes,'order_column_register_sortable');
		$this->loader->add_filter( 'pre_get_posts', $interlocutors_emails_metaboxes,  'set_custom_interlocutors_email_admin_order', 5);
		$this->loader->add_filter( 'manage_interlocutors_email_posts_columns',$interlocutors_emails_metaboxes, 'set_custom_edit_interlocutors_email_columns' );
		$this->loader->add_action( 'manage_interlocutors_email_posts_custom_column' ,$interlocutors_emails_metaboxes, 'custom_interlocutors_email_column', 10, 2 );
		$this->loader->add_action( 'after_delete_post', $interlocutors_emails_metaboxes, 'reorder_interlocutors_email_on_delete' );
		$this->loader->add_action( 'wp_trash_post', $interlocutors_emails_metaboxes, 'reorder_interlocutors_email_on_send_trash' );


	} // define_admin_hooks()

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_public_hooks() {

		$plugin_public = new Alexandria_Mail_Builder_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles', $this->get_version(), TRUE );
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts', $this->get_version(), TRUE );

		$this->loader->add_action('init', $plugin_public, 'register_shortcodes');
		
		// Ajax actions
		add_action('wp_ajax_alexandria_mail_builder_ajax', 'alexandria_mail_builder_ajax');
		add_action('wp_ajax_nopriv_alexandria_mail_builder_ajax', 'alexandria_mail_builder_ajax');

		add_action('wp_ajax_alexandria_email_get_cutom_post_metadata_ajax', 'alexandria_email_get_cutom_post_metadata_ajax');
		add_action('wp_ajax_nopriv_alexandria_email_get_cutom_post_metadata_ajax', 'alexandria_email_get_cutom_post_metadata_ajax');
		
	} // define_public_hooks()

	/**
	 * Register all of the hooks related to the templates.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_template_hooks() {

		$plugin_templates = new Alexandria_Mail_Builder_Template_Functions( $this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('alexandria_mail_builder_main_template', $plugin_templates, 'alexandria_mail_builder_template', 10, 2);
		$this->loader->add_action('alexandria_mail_show_main_template', $plugin_templates, 'alexandria_mail_builder_main_template', 10, 2);

	} // define_template_hooks()

	/**
	 * Register all of the hooks shared between public-facing and admin functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_shared_hooks() {

		$plugin_shared = new Alexandria_Mail_Builder_Shared( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'widgets_init', $plugin_shared, 'widgets_init' );
		$this->loader->add_action( 'save_post_mail_builder', $plugin_shared, 'flush_widget_cache' );
		$this->loader->add_action( 'deleted_post', $plugin_shared, 'flush_widget_cache' );
		$this->loader->add_action( 'switch_theme', $plugin_shared, 'flush_widget_cache' );

	} // define_shared_hooks()


	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since 		1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since 		1.0.0
	 * @return 		string 					The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since 		1.0.0
	 * @return 		Alexandria_Mail_Builder_Loader 		Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since 		1.0.0
	 * @return 		string 					The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	// Option 2

	/**
	 * Register all of the hooks shared between public-facing and admin functionality
	 * of the plugin.
	 *
	 * @since 		1.0.0
	 * @access 		private
	 */
	private function define_widget_hooks() {

		$this->loader->add_action( 'widgets_init', $this, 'widgets_init' );
		$this->loader->add_action( 'deleted_post', $this, 'flush_widget_cache' );
		$this->loader->add_action( 'switch_theme', $this, 'flush_widget_cache' );

	}

	/**
	 * Flushes widget cache
	 *
	 * @since 		1.0.0
	 * @access 		public
	 * @param 		int 		$post_id 		The post ID
	 * @return 		void
	 */
	public function flush_widget_cache( $post_id ) {
	}

	/**
	 * Registers widgets with WordPress
	 *
	 * @since 		1.0.0
	 * @access 		public
	 */
	public function widgets_init() {

		register_widget( 'alexandria_mail_builder_widget' );

	} // widgets_init()

}
